# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: mode_3d_volrat_statistics.py
"""
from metcalcpy.util.mode_arearat_statistics import *
from metcalcpy.util.utils import column_data_by_name_value, THREE_D_DATA_FILTER

__author__ = 'Tatiana Burek'
__version__ = '0.1.0'


def calculate_3d_volrat_fsa_asa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple objects that are forecast

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsa_asa(filtered_data, columns_names_new)


def calculate_3d_volrat_osa_asa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple objects that are observation

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osa_asa(filtered_data, columns_names_new)


def calculate_3d_volrat_asm_asa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple objects that are matched

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """

    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_asm_asa(filtered_data, columns_names_new)


def calculate_3d_volrat_asu_asa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple objects that are unmatched

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_asu_asa(filtered_data, columns_names_new)


def calculate_3d_volrat_fsm_fsa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple forecast objects that are matched

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsm_fsa(filtered_data, columns_names_new)


def calculate_3d_volrat_fsu_fsa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple forecast objects that are unmatched

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsu_fsa(filtered_data, columns_names_new)


def calculate_3d_volrat_osm_osa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple observation objects that are matched

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osm_osa(filtered_data, columns_names_new)


def calculate_3d_volrat_osu_osa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple observation objects that are unmatched

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osu_osa(filtered_data, columns_names_new)


def calculate_3d_volrat_fsm_asm(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple matched objects that are forecasts

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)

    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsm_asm(filtered_data, columns_names_new)


def calculate_3d_volrat_osm_asm(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple matched objects that are observations

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osm_asm(filtered_data, columns_names_new)


def calculate_3d_volrat_osu_asu(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d simple unmatched objects that are observation

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osu_asu(filtered_data, columns_names_new)


def calculate_3d_volrat_fsa_aaa(input_data, columns_names):
    """Performs calculation of Volume-weighted ?

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsa_aaa(filtered_data, columns_names_new)


def calculate_3d_volrat_osa_aaa(input_data, columns_names):
    """Performs calculation of Volume-weighted ?

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osa_aaa(filtered_data, columns_names_new)


def calculate_3d_volrat_fsa_faa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of all 3d forecast objects that are simple

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """

    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsa_faa(filtered_data, columns_names_new)


def calculate_3d_volrat_fca_faa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of all 3d forecast objects that are cluster

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fca_faa(filtered_data, columns_names_new)


def calculate_3d_volrat_osa_oaa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of all 3d observation objects that are simple

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osa_oaa(filtered_data, columns_names_new)


def calculate_3d_volrat_oca_oaa(input_data, columns_names):
    """Performs calculation of Volume-weighted % of all 3d observation objects that are cluster

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_oca_oaa(filtered_data, columns_names_new)


def calculate_3d_volrat_fca_aca(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d cluster objects that are forecast

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fca_aca(filtered_data, columns_names_new)


def calculate_3d_volrat_oca_aca(input_data, columns_names):
    """Performs calculation of Volume-weighted % of 3d cluster objects that are observation

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_oca_aca(filtered_data, columns_names_new)


def calculate_3d_volrat_fsa_osa(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d simple forecasts to
        3d simple observations [frequency bias]

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)

    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsa_osa(filtered_data, columns_names_new)


def calculate_3d_volrat_osa_fsa(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d simple observations to
        3d simple forecasts [1 / frequency bias]

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osa_fsa(filtered_data, columns_names_new)


def calculate_3d_volrat_aca_asa(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d cluster objects to 3d simple objects

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_aca_asa(filtered_data, columns_names_new)


def calculate_3d_volrat_asa_aca(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d simple objects to 3d cluster objects

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_asa_aca(filtered_data, columns_names_new)


def calculate_3d_volrat_fca_fsa(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d cluster forecast objects to
        3d simple forecast objects

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fca_fsa(filtered_data, columns_names_new)


def calculate_3d_volrat_fsa_fca(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d simple forecast objects to
        3d cluster forecast objects

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)

    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_fsa_fca(filtered_data, columns_names_new)


def calculate_3d_volrat_oca_osa(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d cluster observation objects to
        3d simple observation objects

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_oca_osa(filtered_data, columns_names_new)


def calculate_3d_volrat_osa_oca(input_data, columns_names):
    """Performs calculation of Volume Ratio of 3d simple observation objects to
        3d cluster observation objects

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_arearat_osa_oca(filtered_data, columns_names_new)


def calculate_3d_objvhits(input_data, columns_names):
    """Performs calculation of Volume 3d Hits =/2

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)

    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_objahits(filtered_data, columns_names_new)


def calculate_3d_objvmisses(input_data, columns_names):
    """Performs calculation of Volume 3d Misses = OSU

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_objamisses(filtered_data, columns_names_new)


def calculate_3d_objvfas(input_data, columns_names):
    """Performs calculation of Volume 3d False Alarms = FSU

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_objafas(filtered_data, columns_names_new)


def calculate_3d_objvcsi(input_data, columns_names):
    """Performs calculation of Volume  3d critical success index CSI = hits //2 + OSU + FSU]

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_objacsi(filtered_data, columns_names_new)


def calculate_3d_objvpody(input_data, columns_names):
    """Performs calculation of Volume  3d prob of detecting yes PODY = hits //2 + OSU]

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_objapody(filtered_data, columns_names_new)


def calculate_3d_objvfar(input_data, columns_names):
    """Performs calculation of Volume 3d FAR = false alarms //2 + FSU]

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated statistic
            or None if some of the data values are missing or invalid
    """
    columns_names_new = rename_column(columns_names)
    filtered_data = column_data_by_name_value(input_data, columns_names_new, THREE_D_DATA_FILTER)
    return calculate_objafar(filtered_data, columns_names_new)


def rename_column(columns_names):
    """Change the column name array element from 'volume' to 'area'

        Args:
            columns_names: array of column names

        Returns:
            array with changed column name
    """
    columns_names_new = []
    for index, name in enumerate(columns_names):
        if name == 'volume':
            columns_names_new.insert(index, 'area')
        else:
            columns_names_new.insert(index, name)
    return columns_names_new



import pytest
import numpy as np
import metcalcpy.util.utils as utils

def test_convert_lon_indices_working():

    # Verify that some values were correctly converted
    # and that the ordering in the array is from negative to positive
    np_lon = np.linspace(0, 359, 360)
    minlon_in = -180
    range_in = 360
    west_east, west_east_indices = utils.convert_lons_indices(np_lon, minlon_in, range_in)
    assert west_east[0] == -180.0
    assert west_east[359] == 179.0


if __name__ == "__main__":
    test_convert_lon_indices_working()



"""Tests the operation of METcalcpy's event_equalize_against_values code."""

import pandas as pd

from metcalcpy.event_equalize_against_values import event_equalize_against_values


def test_event_equalize_against_values():
    """Tests event equalization against values."""

    indy_var = "fcst_lead"
    series_val = dict({'model': ["AFWAOCv3.5.1_d01", "NoahMPv3.5.1_d01"]})

    fcst_var_val = dict({'APCP_03': ["RATIO_FSA_ASA"]})
    input_data_file = 'data/ee_av_input.data'
    stats_input_data_file = 'data/stats_ee_av_input.data'
    output_data_file = 'data/ee_av_output_py.data'

    # read the input data file into a data frame
    input_data = pd.read_csv(input_data_file, header=[0], sep='\t')
    stats_data = pd.read_csv(stats_input_data_file, header=[0], sep='\t')
    output_data = pd.DataFrame()

    for fcst_var, fcst_var_stats in fcst_var_val.items():
        for series_var, series_var_vals in series_val.items():
            # ungroup series value
            series_var_vals_no_group = []
            for val in series_var_vals:
                split_val = val.split(',')
                series_var_vals_no_group.extend(split_val)

            ee_stats_equalize = input_data[
                (input_data['fcst_var'] == fcst_var)
                & (input_data[series_var].isin(series_var_vals_no_group))
                ]
            f_plot = stats_data[(stats_data['fcst_var'] == fcst_var)
                                & (stats_data[series_var].isin(series_var_vals_no_group))
                                ]
            ee_stats_equalize_unique = (list(set(ee_stats_equalize['equalize'])))
            f_plot = event_equalize_against_values(f_plot, ee_stats_equalize_unique)

            # append EE data to result
            if output_data.empty:
                output_data = f_plot
            else:
                output_data.append(f_plot)
        # save to file
        output_data.to_csv(index=False, sep='\t', path_or_buf=output_data_file)


if __name__ == "__main__":
    test_event_equalize_against_values()



import setuptools
from setuptools import setup, find_packages
from distutils.util import convert_path

with open("README.md", "r") as fh:
    long_description = fh.read()

main_ns = {}
version_path = convert_path('docs/version')
with open(version_path) as version_file:
    exec(version_file.read(), main_ns)

setuptools.setup(
    name="metcalcpy",
    version=main_ns['__version__'],
    author="METplus",
    author_email="met-help@ucar.edu",
    description="statistics and util package for METplus",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/dtcenter/METcalcpy",
    packages=setuptools.find_packages(),
    classifiers=[
         "Programming Language :: Python :: 3",
         "License :: OSI Approved :: Apache Software License",
         "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
)



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: nbrcnt_statistics.py
"""
import warnings

from metcalcpy.util.utils import round_half_up, sum_column_data_by_name, PRECISION, get_total_values

__author__ = 'Tatiana Burek'
__version__ = '0.1.0'


def calculate_nbr_fbs(input_data, columns_names, aggregation=False):
    """Performs calculation of NBR_FBS - Fractions Brier Score

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated NBR_FBS as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        fbs = sum_column_data_by_name(input_data, columns_names, 'fbs') / total
        result = round_half_up(fbs, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_nbr_fss(input_data, columns_names, aggregation=False):
    """Performs calculation of NBR_FSS - Fractions Skill Score

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated NBR_FSS as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        fss_den = sum_column_data_by_name(input_data, columns_names, 'fss') / total
        fbs = sum_column_data_by_name(input_data, columns_names, 'fbs') / total
        fss = 1.0 - fbs / fss_den
        result = round_half_up(fss, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_nbr_afss(input_data, columns_names, aggregation=False):
    """Performs calculation of NBR_AFSS - Asymptotic Fractions Skill Score

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated NBR_AFSS as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        f_rate = sum_column_data_by_name(input_data, columns_names, 'f_rate') / total
        o_rate = sum_column_data_by_name(input_data, columns_names, 'o_rate') / total

        afss_num = 2.0 * f_rate * o_rate
        afss_den = f_rate * f_rate + o_rate * o_rate
        afss = afss_num / afss_den
        result = round_half_up(afss, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_nbr_ufss(input_data, columns_names, aggregation=False):
    """Performs calculation of NBR_UFSS - Uniform Fractions Skill Score

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated NBR_UFSS as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        o_rate = sum_column_data_by_name(input_data, columns_names, 'o_rate') / total
        ufss = 0.5 + o_rate / 2.0
        result = round_half_up(ufss, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_nbr_f_rate(input_data, columns_names, aggregation=False):
    """Performs calculation of NBR_F_RATE - Forecast event frequency

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated NBR_F_RATE as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        f_rate = sum_column_data_by_name(input_data, columns_names, 'f_rate') / total
        result = round_half_up(f_rate, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_nbr_o_rate(input_data, columns_names, aggregation=False):
    """Performs calculation of NBR_O_RATE - Observed event frequency

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated NBR_O_RATE as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        o_rate = sum_column_data_by_name(input_data, columns_names, 'o_rate') / total
        result = round_half_up(o_rate, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_nbr_cnt_total(input_data, columns_names):
    """Performs calculation of Total number of matched pairs for Neighborhood Continuous Statistics
        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated Total number of matched pairs as float
            or None if some of the data values are missing or invalid
    """
    total = sum_column_data_by_name(input_data, columns_names, 'total')
    return round_half_up(total, PRECISION)



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ** University of Illinois, Urbana-Champaign
 # ============================*
 
 
 
import os
import numpy as np
import datetime
import bisect
from scipy import stats
from scipy.signal import argrelextrema
#from metplus.util import config_metplus, get_start_end_interval_times, get_lead_sequence
#from metplus.util import get_skip_times, skip_time, is_loop_by_init, ti_calculate
from metcalcpy.contributed.blocking_weather_regime.Blocking_WeatherRegime_util import read_nc_met


class BlockingCalculation():
    """Contains the programs to calculate Blocking via the Pelly-Hoskins Method
    """
    def __init__(self,label):

        self.blocking_anomaly_var = os.environ.get(label+'_BLOCKING_ANOMALY_VAR','Z500_ANA')
        self.blocking_var = os.environ.get(label+'_BLOCKING_VAR','Z500')
        self.smoothing_pts = int(os.environ.get(label+'_SMOOTHING_PTS',9))
        lat_delta_in = os.environ.get(label+'_LAT_DELTA','-5,0,5')
        self.lat_delta = list(map(int,lat_delta_in.split(",")))
        self.n_s_limits = int(os.environ.get(label+'_NORTH_SOUTH_LIMITS',30))
        self.ibl_dist = int(os.environ.get(label+'_IBL_DIST',7))
        self.ibl_in_gibl = int(os.environ.get(label+'_IBL_IN_GIBL',15))
        self.gibl_overlap = int(os.environ.get(label+'_GIBL_OVERLAP',10))
        self.block_time = int(os.environ.get(label+'_BLOCK_TIME',5))  ###Should fix so it supports other times"
        self.block_travel = int(os.environ.get(label+'_BLOCK_TRAVEL',45))
        self.block_method = os.environ.get(label+'_BLOCK_METHOD','PH')

        # Check data requirements
        if self.smoothing_pts % 2 == 0:
            print('ERROR: Smoothing Radius must be an odd number given in grid points, Exiting...')
            exit()


    def run_CBL(self,cblinfiles,nseasons,dseasons):

        z500_anom_4d,lats,lons,timedict = read_nc_met(cblinfiles,self.blocking_anomaly_var,nseasons,dseasons)

        #Create Latitude Weight based for NH
        cos = lats
        cos = cos * np.pi / 180.0
        way = np.cos(cos)
        way = np.sqrt(way)
        weightf = np.repeat(way[:,np.newaxis],360,axis=1)

        ####Find latitude of maximum high-pass STD (CBL)
        yrlen = len(z500_anom_4d[:,0,0,0])
        mstd = np.nanstd(z500_anom_4d,axis=1)
        mhweight = mstd * weightf
        cbli = np.argmax(mhweight,axis=1)
        CBL = np.zeros((yrlen,len(lons)))
        for j in np.arange(0,yrlen,1):
            CBL[j,:] = lats[cbli[j,:]]

        ###Apply Moving Average to Smooth CBL Profiles
        lt = len(lons)
        CBLf = np.zeros((yrlen,len(lons)))
        m=int((self.smoothing_pts-1)/2.0)
        for i in np.arange(0,len(CBL[0,:]),1):
            ma_indices = np.arange(i-m,i+m+1)
            ma_indices = np.where(ma_indices >= lt,ma_indices-lt,ma_indices)
            CBLf[:,i] = np.nanmean(CBL[:,ma_indices],axis=1).astype(int)
        
        return CBLf,lats,lons,mhweight,timedict


    def run_mod_blocking1d(self,a,cbl,lat,lon,meth):
        lat_d = self.lat_delta
        dc = (90 - cbl).astype(int)
        db = self.n_s_limits
        BI = np.zeros((len(a[:,0,0]),len(lon)))
        blon = np.zeros((len(a[:,0,0]),len(lon)))
        if meth=='PH':
            # loop through days
            for k in np.arange(0,len(a[:,0,0]),1):
                blontemp=0
                q=0
                BI1=np.zeros((len(lat_d),len(lon)))
                for l in lat_d:
                    blon1 = np.zeros(len(lon))
                    d0 = dc-l
                    dn = ((dc - 1*db/2) - l).astype(np.int64)
                    ds = ((dc + 1*db/2) - l).astype(np.int64)
                    GHGS = np.zeros(len(cbl))
                    GHGN = np.zeros(len(cbl))
                    for jj in np.arange(0,len(cbl),1):
                        GHGN[jj] = np.mean(a[k,dn[jj]:d0[jj]+1,jj])
                        GHGS[jj] = np.mean(a[k,d0[jj]:ds[jj]+1,jj])
                    BI1[q,:] = GHGN-GHGS
                    q = q +1
                BI1 = np.max(BI1,axis=0)
                block = np.where((BI1>0))[0]
                blon1[block]=1
                blontemp = blontemp + blon1
                BI[k,:] = BI1
                blon[k,:] = blontemp

        return blon,BI


    def run_Calc_IBL(self,cbl,iblinfiles,nseasons, dseasons):

        z500_daily,lats,lons,timedict = read_nc_met(iblinfiles,self.blocking_var,nseasons,dseasons)

        #Initilize arrays for IBLs and the blocking index
        # yr, day, lon
        yrlen = len(z500_daily[:,0,0,0])
        blonlong = np.zeros((yrlen,len(z500_daily[0,:,0,0]),len(lons)))
        BI = np.zeros((yrlen,len(z500_daily[0,:,0,0]),len(lons)))

        #Using long-term mean CBL and acsessing module of mod.py
        cbl = np.nanmean(cbl,axis=0)
        for i in np.arange(0,yrlen,1):
            blon,BI[i,:,:] = self.run_mod_blocking1d(z500_daily[i,:,:,:],cbl,lats,lons,self.block_method)
            blonlong[i,:,:] = blon

        return blonlong,timedict


    def run_Calc_GIBL(self,ibl,lons):

        #Initilize GIBL Array
        GIBL = np.zeros(np.shape(ibl))

        #####Loop finds IBLs within 7 degree of each other creating one group. Finally
        ##### A GIBL exist if it has more than 15 IBLs
        crit = self.ibl_in_gibl

        for i in np.arange(0,len(GIBL[:,0,0]),1):
            for k in np.arange(0,len(GIBL[0,:,0]),1):
                gibli = np.zeros(len(GIBL[0,0,:]))
                thresh = crit/2.0
                a = ibl[i,k,:]
                db = self.ibl_dist
                ibli = np.where(a==1)[0]
                if len(ibli)==0:
                    continue
                idiff = ibli[1:] - ibli[:-1]
                bt=0
                btlon = ibli[0]
                ct = 1
                btfin = []
                block = ibli
                block = np.append(block,block+360)
                for ll in np.arange(1,len(block),1):
                    diff = np.abs(block[ll] - block[ll-1])
                    if diff == 1:
                        bt = [block[ll]]
                        btlon = np.append(btlon,bt)
                        ct = ct + diff
                    if diff <= thresh and diff != 1:
                        bt = np.arange(block[ll-1]+1,block[ll]+1,1)
                        btlon = np.append(btlon,bt)
                        ct = ct + diff
                    if diff > thresh or ll==(len(block)-1):
                        if ct >= crit:
                            btfin = np.append(btfin,btlon)
                            ct=1
                        ct = 1
                        btlon = block[ll]
                if len(btfin)/2 < crit :
                    btfin = []
                if len(btfin)==0:
                    continue
                gibl1 = btfin
                temp = np.where(gibl1>=360)[0]
                gibl1[temp] = gibl1[temp] - 360
                gibli[gibl1.astype(int)] = 1
                GIBL[i,k,:] = gibli

        return GIBL


    def Remove(self,duplicate):
        final_list = []
        for num in duplicate:
            if num not in final_list:
                final_list.append(num)
        return final_list


    def run_Calc_Blocks(self,ibl,GIBL,lon,tsin):

        crit = self.ibl_in_gibl

        ##Count up the blocked longitudes for each GIBL
        c = np.zeros((GIBL.shape))
        lonlen = len(lon)
        sz = []
        mx = []
        min = []

        for y in np.arange(0,len(GIBL[:,0,0]),1):
            for k in np.arange(0,len(GIBL[0,:,0]),1):
                a = GIBL[y,k] # Array of lons for each year,day
                ct=1
                ai = np.where(a==1)[0]
                ai = np.append(ai,ai+360)
                temp = np.where(ai>=360)[0]
                bi=list(ai)
                bi = np.array(bi)
                bi[temp] = bi[temp]-360
                # Loop through the lons that are part of the GIBL
                for i in np.arange(0,len(ai)-1,1):
                    diff = ai[i+1] - ai[i]
                    c[y,k,bi[i]] = ct
                    if diff==1:
                        ct=ct+1
                    else:
                        sz = np.append(sz,ct)
                        ct=1

        ########## - finding where the left and right limits of the block are - ################
        for i in np.arange(0,len(c[:,0,0]),1):
            for k in np.arange(0,len(c[0,:,0]),1):
                maxi = argrelextrema(c[i,k],np.greater,mode='wrap')[0]
                mini = np.where(c[i,k]==1)[0]
                if c[i,k,lonlen-1]!=0 and c[i,k,0]!=0:
                    mm1 = mini[-1]
                    mm2 = mini[:-1]
                    mini = np.append(mm1,mm2)
                mx = np.append(mx,maxi)
                min = np.append(min,mini)

        locy, locd, locl = np.where(c==crit)

        A = np.zeros(lonlen)
        A = np.expand_dims(A,axis=0)

        ################# - Splitting up each GIBL into its own array - ###################

        for i in np.arange(0,len(locy),1):
            m = locy[i]   #year
            n = locd[i]   #day
            o = locl[i]   #long where 15
            mm = int(mx[i])
            mn = min[i]
            temp1 = GIBL[m,n]
            temp2 = np.zeros(lonlen)
            if mn>mm:
                diff = int(mm - c[m,n,mm] + 1)
                lons = lon[diff]
                place1 = np.arange(lons,lonlen,1)
                place2 = np.arange(0,mm+1,1)
                bl = np.append(place2,place1).astype(int)
            if temp1[lonlen-1] ==1 and mm>200:
                lons = lon[mm]
                beg = mm - c[m,n,mm] + 1
                bl = np.arange(beg,mm+1,1).astype(int)
            if mm>mn: #temp1[359] ==0:
                lons = lon[mm]
                beg = mm - c[m,n,mm] + 1
                bl = np.arange(beg,mm+1,1).astype(int)
            temp2[bl] = 1
            temp2 = np.expand_dims(temp2,axis=0)
            A = np.concatenate((A,temp2),axis=0)

        A = A[1:]

        ######### - Getting rid of non-consectutve Time steps which would prevent blocking - #################
        dd=[]
        dy = []
        dA = A[0]
        dA = np.expand_dims(dA,axis=0)
        ct=0
        for i in np.arange(1,len(locy),1):
            dd1 = locd[i-1]
            dd2 = locd[i]
            if dd2-dd1 > 2:
                ct = 0
                continue
            if ct == 0:
                dd = np.append(dd,locd[i-1])
                dy = np.append(dy,locy[i-1])
                temp2 = np.expand_dims(A[i-1],axis=0)
                dA = np.concatenate((dA,temp2),axis=0)
                ct = ct + 1
            if dd2-dd1<=2:
                dd=np.append(dd,locd[i])
                dy = np.append(dy,locy[i])
                temp2 = np.expand_dims(A[i],axis=0)
                dA = np.concatenate((dA,temp2),axis=0)
                ct = ct + 1

        dA=dA[1:]
        dAfin = dA

        ############ - Finding center longitude of block - ##############
        middle=[]
        for l in np.arange(0,len(dAfin),1):
            temp = np.where(dAfin[l]==1)[0]
            if len(temp) % 2 == 0:
                temp = np.append(temp,0)
            midtemp = np.median(temp)
            middle = np.append(middle,midtemp)


        #####Track blocks. Makes sure that blocks overlap with at least 10 longitude points on consecutive
        overlap = self.gibl_overlap
        btime = self.block_time
        fin = [[]]
        finloc = [[]]
        ddcopy=dd*1.0
        noloc=[]
        failloc = [[]]
        for i in np.arange(0,len(c[:,0,0]),1):
            yri = np.where(dy==i)[0]
            B = [[]]
            ddil =1.0 * ddcopy[yri]
            dyy = np.where(dy==i)[0]
            rem = []
            for dk in ddil:
                if len(ddil) < btime:
                    continue
                ddil = np.append(ddil[0]-1,ddil)
                diff = np.diff(ddil)
                diffB=[]
                dB =1
                cnt = 1
                while dB<=2:
                    diffB = np.append(diffB,ddil[cnt])
                    dB = diff[cnt-1]
                    if ddil[cnt]==ddil[-1]:
                        dB=5
                    cnt=cnt+1
                diffB = np.array(self.Remove(diffB))
                locb = []
                for ll in diffB:
                    locb = np.append(locb,np.where((dy==i) & (dd==ll))[0])
                ddil=ddil[1:]
                locbtemp = 1.0*locb
                ree=np.empty(0,dtype=int)
                for hh in np.arange(0,len(noloc),1):
                    ree = np.append(ree,np.where(locb == noloc[hh])[0])
                ree.astype(int)
                locbtemp = np.delete(locbtemp,ree)
                locb=locbtemp * 1.0
                datemp = dAfin[locb.astype(int)]
                blocktemp = [[datemp[0]]]
                locbi = np.array([locb[0]])
                ll1=0
                pass1 = 0
                ai=[0]
                add=0
                for ll in np.arange(0,len(locb)-1,1):
                    if ((dd[locb[ll+1].astype(int)] - dd[locb[ll1].astype(int)]) >=1) & ((dd[locb[ll+1].astype(int)] - dd[locb[ll1].astype(int)]) <=2):
                        add = datemp[ll1] + datemp[ll+1]
                    ai = np.where(add==2)[0]
                    if len(ai)>overlap:
                        locbi=np.append(locbi,locb[ll+1])
                        ll1=ll+1
                        add=0
                    if (len(ai)<overlap):
                        add=0
                        continue
                if len(locbi)>4:
                    noloc = np.append(noloc,locbi)
                    finloc = finloc + [locbi]
                    for jj in locbi:
                        rem = np.append(rem,np.where(dyy==jj)[0])
                    ddil = np.delete(ddcopy[yri],rem.astype(int))
                if len(locbi)<=4:
                    noloc = np.append(noloc,locbi)
                    if len(locbi)<=2:
                        failloc=failloc+[locbi]
                    for jj in locbi:
                        rem = np.append(rem,np.where(dyy==jj)[0])
                    ddil = np.delete(ddcopy[yri],rem.astype(int))

        blocks = finloc[1:]
        noblock = failloc[1:]

        ############Get rid of blocks that travel downstream##########
        ######If center of blocks travel downstream further than 45 degrees longitude, 
        ######cancel block moment it travels out of this limit
        newblock = [[]]
        newnoblock=[[]]
        distthresh = self.block_travel
        for bb in blocks:
            diffb = []
            start = middle[bb[0].astype(int)]
            for bbs in bb:
                diffb = np.append(diffb, start - middle[bbs.astype(int)])
            loc = np.where(np.abs(diffb) > distthresh)[0]
            if len(loc)==0:
                newblock = newblock +[bb]
            if len(loc)>0:
                if len(bb[:loc[0]]) >4:
                    newblock = newblock + [bb[:loc[0]]]
                if len(bb[:loc[0]]) <=2:
                    noblock = noblock + [bb]

        blocks = newblock[1:]

        #Create a final array with blocking longitudes. Similar to IBL/GIBL, but those that pass the duration threshold
        blockfreq = np.zeros((len(ibl[:,0,0]),len(ibl[0,:,0]),360))
        savecbl=[]
        savemiddle = []
        saveyr=[]
        numblock=0
        for i in np.arange(0,len(blocks),1):
            temp = blocks[i]
            numblock=numblock+1
            for j in temp:
                j=int(j)
                daycomp = int(dd[j])
                yearcomp = int(dy[j])
                saveyr = np.append(saveyr,dy[j])
                middlecomp = middle[j].astype(int)
                mc1 = int(round(middlecomp / 2.5))
                blockfreq[yearcomp,daycomp] = blockfreq[yearcomp,daycomp] + dAfin[j]
                ct = ct + 1

        return blockfreq



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: pstd_statistics.py
"""
import warnings
import numpy as np
import pandas as pd

from metcalcpy.util.met_stats import get_column_index_by_name
from metcalcpy.util.utils import round_half_up, sum_column_data_by_name, PRECISION

__author__ = 'Tatiana Burek'
__version__ = '0.1.0'


def calculate_pstd_brier(input_data, columns_names):
    """Performs calculation of PSTD_BRIER - Brier Score

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated PSTD_BRIER as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        df_pct_perm = _calc_common_stats(columns_names, input_data)
        t_table = df_pct_perm['n_i'].sum()
        o_bar_table = df_pct_perm['oy_i'].sum() / t_table
        o_bar = input_data[0, get_column_index_by_name(columns_names, 'o_bar')]

        reliability = calc_reliability(t_table, df_pct_perm)
        resolution = calc_resolution(t_table, df_pct_perm, o_bar)
        uncertainty = calc_uncertainty(o_bar_table)

        brier = reliability - resolution + uncertainty
        result = round_half_up(brier, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_bss_smpl(input_data, columns_names):
    """Performs calculation of BSS_SMPL - Brier Skill Score relative to sample climatology

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated BSS_SMPL as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        df_pct_perm = _calc_common_stats(columns_names, input_data)
        t_table = df_pct_perm['n_i'].sum()
        o_bar_table = df_pct_perm['oy_i'].sum() / t_table
        o_bar = input_data[0, get_column_index_by_name(columns_names, 'o_bar')]

        reliability = calc_reliability(t_table, df_pct_perm)
        resolution = calc_resolution(t_table, df_pct_perm, o_bar)
        uncertainty = calc_uncertainty(o_bar_table)

        bss_smpl = (resolution - reliability) / uncertainty
        result = round_half_up(bss_smpl, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_baser(input_data, columns_names):
    """Performs calculation of BASER - The Base Rate
        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated BASER as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:

        df_pct_perm = _calc_common_stats(columns_names, input_data)
        t_table = df_pct_perm['n_i'].sum()
        baser = df_pct_perm['oy_i'].sum() / t_table
        result = round_half_up(baser, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_reliability(input_data, columns_names):
    """Performs calculation of RELIABILITY - Reliability

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated RELIABILITY as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        df_pct_perm = _calc_common_stats(columns_names, input_data)
        t_table = df_pct_perm['n_i'].sum()

        reliability = calc_reliability(t_table, df_pct_perm)
        result = round_half_up(reliability, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_resolution(input_data, columns_names):
    """Performs calculation of RESOLUTION - Resolution

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated RESOLUTION as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        df_pct_perm = _calc_common_stats(columns_names, input_data)
        o_bar = input_data[0, get_column_index_by_name(columns_names, 'o_bar')]
        t_table = df_pct_perm['n_i'].sum()

        resolution = calc_resolution(t_table, df_pct_perm, o_bar)
        result = round_half_up(resolution, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_uncertainty(input_data, columns_names):
    """Performs calculation of UNCERTAINTY - Uncertainty

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated UNCERTAINTY as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        df_pct_perm = _calc_common_stats(columns_names, input_data)
        t_table = df_pct_perm['n_i'].sum()
        o_bar_table = df_pct_perm['oy_i'].sum() / t_table

        uncertainty = calc_uncertainty(o_bar_table)
        result = round_half_up(uncertainty, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_calibration(input_data, columns_names):
    """Performs calculation of calibration

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated calibration as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        oy_i = sum(input_data[:, get_column_index_by_name(columns_names, 'oy_i')])
        n_i = calculate_pstd_ni(input_data, columns_names)
        calibration = oy_i / n_i
        result = round_half_up(calibration, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_ni(input_data, columns_names):
    """Performs calculation of ni - Uncertainty

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated ni as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        oy_i = sum(input_data[:, get_column_index_by_name(columns_names, 'oy_i')])
        on_i = sum(input_data[:, get_column_index_by_name(columns_names, 'on_i')])
        n_i = oy_i + on_i
        result = round_half_up(n_i, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pstd_roc_auc(input_data, columns_names):
    """Performs calculation of ROC_AUC - Area under the receiver operating characteristic curve

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated ROC_AUC as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        df_pct_perm = _calc_common_stats(columns_names, input_data)
        roc = _calc_pct_roc(df_pct_perm)
        # add 1st and last rows
        final_roc = pd.DataFrame(
            {'thresh': 0, 'n11': 0, 'n10': 0, 'n01': 0, 'n00': 0, 'pody': 1, 'pofd': 1},
            index=[0])
        final_roc = final_roc.append(roc, ignore_index=True)
        final_roc = final_roc.append(
            pd.DataFrame(
                {'thresh': 0, 'n11': 0, 'n10': 0, 'n01': 0, 'n00': 0, 'pody': 0, 'pofd': 0},
                index=[0]),
            ignore_index=True)

        roc_auc = 0
        for index, row in final_roc.iterrows():
            if index != 0:
                roc_auc = roc_auc + 0.5 * (final_roc.iloc[index - 1]['pody'] + row.pody) \
                          * (final_roc.iloc[index - 1]['pofd'] - row.pofd)

        result = round_half_up(roc_auc, PRECISION)

    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calc_uncertainty(o_bar_table):
    """Performs calculation of uncertainty
         Args: o_bar_table
         Returns: uncertainty
    """
    uncertainty = o_bar_table * (1 - o_bar_table)
    return uncertainty


def calc_resolution(t_table, df_pct_perm, o_bar):
    """Performs calculation of resolution
         Args: t_table
            df_pct_perm
            o_bar

         Returns: resolution
    """
    resolution = sum([row.n_i * (row.o_bar_i - o_bar) * (row.o_bar_i - o_bar)
                      for index, row in df_pct_perm.iterrows()]) \
                 / t_table
    return resolution


def calc_reliability(t_table, df_pct_perm):
    """Performs calculation of reliability
         Args: t_table
            df_pct_perm
            o_bar

         Returns: reliability
    """
    reliability = sum([row.n_i * (row.thresh_i - row.o_bar_i) * (row.thresh_i - row.o_bar_i)
                       for index, row in df_pct_perm.iterrows()]) \
                  / t_table
    return reliability


def _calc_common_stats(columns_names, input_data):
    """ Creates a data frame to hold the aggregated contingency table and ROC data
            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

            Returns:
                pandas data frame with columns:
                    - thresh_i
                    - oy_i
                    - on_i
                    - n_i
                    - o_bar_i
    """
    pct_perm = {'thresh_i': [], 'oy_i': [], 'on_i': []}
    for column in columns_names:
        index = get_column_index_by_name(columns_names, column)

        if "oy_i" in column:
            sum_val = sum_column_data_by_name(input_data, columns_names, column)
            pct_perm['oy_i'].append(sum_val)
        elif "on_i" in column:
            sum_val = sum_column_data_by_name(input_data, columns_names, column)
            pct_perm['on_i'].append(sum_val)
        elif 'thresh_i' in column:
            pct_perm['thresh_i'].append(input_data[0, index])
    # calculate vectors and constants to use below
    df_pct_perm = pd.DataFrame(pct_perm)
    n_i = [row.oy_i + row.on_i for index, row in df_pct_perm.iterrows()]
    df_pct_perm['n_i'] = n_i

    # use only records with n_i != 0
    df_pct_perm = df_pct_perm[df_pct_perm['n_i'] != 0]
    o_bar_i = [row.oy_i / row.n_i for index, row in df_pct_perm.iterrows()]
    calibration_i = [row.oy_i / row.n_i for index, row in df_pct_perm.iterrows()]
    df_pct_perm['o_bar_i'] = o_bar_i
    return df_pct_perm


def _calc_pct_roc(data):
    """ Creates a data frame to hold the aggregated contingency table and ROC data
            Args:
                data: pandas data frame with pstd data and column names:
                     - thresh_i
                    - oy_i
                    - on_i
                    - n_i
                    - o_bar_i

            Returns:
                pandas data frame with ROC data and columns:
                - thresh
                - n11
                - n10
                - n01
                - n00
                - pody
                - pofd
    """
    # create a data frame to hold the aggregated contingency table and ROC data
    list_thresh = np.unique(np.sort(data['thresh_i'].to_numpy()))
    df_roc = pd.DataFrame(
        {'thresh': list_thresh, 'n11': None, 'n10': None,
         'n01': None, 'n00': None, 'pody': None, 'pofd': None})

    # build the ROC contingency data table
    for thresh in list_thresh:
        is_bigger = data['thresh_i'] > thresh
        df_roc.at[df_roc.index[df_roc["thresh"] == thresh], 'n11'] = sum(data[is_bigger]['oy_i'])
        df_roc.at[df_roc.index[df_roc["thresh"] == thresh], 'n10'] = sum(data[is_bigger]['on_i'])

        is_less = data['thresh_i'] <= thresh
        df_roc.at[df_roc.index[df_roc["thresh"] == thresh], 'n01'] = sum(data[is_less]['oy_i'])
        df_roc.at[df_roc.index[df_roc["thresh"] == thresh], 'n00'] = sum(data[is_less]['on_i'])

    # generate the pody and pofd scores from the contingency tables
    df_roc['pody'] = [row.n11 / (row.n11 + row.n01) for index, row in df_roc.iterrows()]
    df_roc['pofd'] = [row.n10 / (row.n10 + row.n00) for index, row in df_roc.iterrows()]

    return df_roc


def calculate_pct_total(input_data, columns_names):
    """Performs calculation of Total number of matched pairs for
        Contingency Table Counts for Probabilistic forecasts
        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated Total number of matched pairs as float
            or None if some of the data values are missing or invalid
    """
    total = sum_column_data_by_name(input_data, columns_names, 'total')
    return round_half_up(total, PRECISION)



import os
import sys
import argparse
import logging
import numpy as np
import xarray as xr


def read_tcrmw(filename):
    ds = xr.open_dataset(filename)
    # range, azimuth, pressure, track_point
    for var in ds.keys():
        logging.info((var, ds[var].dims))
    for coord in ds.coords:
        logging.debug((coord, ds[coord].values))

    return ds


def compute_interpolation_weights(args, ds, levels):
    dims = ds[args.T].dims
    logging.info(dims)
    nr, na, nl, nt = ds[args.T].shape
    logging.info((nr, na, nl, nt))


def compute_wind_components(args, ds):
    """
    e_r = cos(theta) e_x + sin(theta) e_y
    e_theta = - sin(theta) e_x + cos(theta) e_y
    """
    nr, na, nl, nt = ds[args.T].shape
    logging.info((nr, na, nl, nt))
    theta = np.empty((nr, na, nl, nt), dtype=np.float32)
    for i in range(nr):
        for k in range(nl):
            for t in range(nt):
                theta[i, :, k, t] \
                    = (np.pi / 180) * ds['azimuth'].values + np.pi / 2
    mask = np.greater(theta, 2 * np.pi)
    theta[mask] = theta[mask] - 2 * np.pi
    u_radial = np.cos(theta) * ds[args.u].values + np.sin(theta) * ds[args.v].values
    u_tangential = - np.sin(theta) * ds[args.u].values + np.cos(theta) * ds[args.v].values
    return u_radial, u_tangential


def test_plot_pressure_lev(args, ds, track_index=0):
    # Plot setup
    import matplotlib.pyplot as plt
    #import seaborn as sns
    # Set for dark PyCharm theme
    #plt.style.use('dark_background')
    textcolor = (175 / 255, 177 / 255, 179 / 255)
#    facecolor = (60 / 255, 63 / 255, 65 / 255)
    # textcolor = (0, 0, 0)
    # facecolor = (1, 1, 1)
    #sns.set_context('notebook')
    plt.rcParams['figure.dpi'] = 300
    plt.rcParams['text.color'] = textcolor
    plt.rcParams['axes.edgecolor'] = textcolor
    plt.rcParams['axes.labelcolor'] = textcolor
    plt.rcParams['xtick.color'] = textcolor
    plt.rcParams['ytick.color'] = textcolor
    fig = plt.figure(figsize=(10, 5))
    #fig.patch.set_facecolor(facecolor)
    ax = fig.add_subplot()
    #ax.set_facecolor(facecolor)
    ax.annotate('Tangential Wind (m s-1)', xy=(14, 350), color='darkgreen')
    ax.annotate('Temperature (K)', xy=(14, 370), color='darkblue')
    # nautical miles to kilometers conversion factor
    nm_to_km = 1.852
    ax.set_xlabel(
        'Range (RMW = %4.1f km)' % (nm_to_km * ds['RMW'][track_index]))
    ax.set_xticks(np.arange(1, 20))
    ax.set_ylabel('Pressure (mb)')
    ax.set_yscale('symlog')
    ax.set_ylim(1000, 250)
    ax.set_yticks(np.arange(1000, 250, -100))
    ax.set_yticklabels(np.arange(1000, 250, -100))

    # Azimuthal averages
    u_radial_azi_mean = np.mean(ds['u_radial'].values, axis=1)
    u_tangential_azi_mean = np.mean(ds['u_tangential'].values, axis=1)
    T_azi_mean = np.mean(ds[args.T].values, axis=1)

    # Contour plots
    u_contour = ax.contour(ds['range'].values, ds['pressure'].values,
        u_tangential_azi_mean[:, :, track_index].transpose(),
        levels=np.arange(5, 40, 5), colors='darkgreen', linewidths=1)
    ax.clabel(u_contour, colors='darkgreen', fmt='%1.0f')
    T_contour = ax.contour(ds['range'].values, ds['pressure'].values,
        T_azi_mean[:, :, track_index].transpose(),
        levels=np.arange(250, 300, 10), colors='darkblue', linewidths=1)
    ax.clabel(T_contour, colors='darkblue', fmt='%1.0f')

    # plt.savefig(os.path.join(args.datadir, 'test.pdf'))
    # plt.savefig(os.path.join(args.datadir, 'test.png'), dpi=300)
    plt.show()


if __name__ == '__main__':
    """
    Parse command line arguments
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('--datadir', type=str,
                        default=os.getenv('DATA_DIR'),
                        help='top-level data directory (default $DATA_DIR)')
    parser.add_argument('--input', type=str,
                        required=True,
                        help='input file name')
    parser.add_argument('--output', type=str,
                        required=True,
                        help='output file name')
    parser.add_argument('--logfile', type=str,
                        default=sys.stdout,
                        help='log file (default stdout)')
    parser.add_argument('--debug', action='store_true',
                        help='set logging level to debug')
    parser.add_argument('--u', type=str,
                        help='zonal wind field',
                        default='UGRD')
    parser.add_argument('--v', type=str,
                        help='meridional wind field',
                        default='VGRD')
    parser.add_argument('--T', type=str,
                        help='temperature field',
                        default='TMP')
    parser.add_argument('--RH', type=str,
                        help='relative humidity field',
                        default='RH')
    parser.add_argument('--vars', type=str,
                        help='additional variables to process',
                        default=',')
    parser.add_argument('--levels', type=str,
                        help='vertical height levels',
                        default='100,200,500,1000,1500,2000,3000,4000,5000')
    args = parser.parse_args()

    """
    Setup logging
    """
    logging_level = logging.DEBUG if args.debug else logging.INFO
    logging.basicConfig(stream=args.logfile, level=logging_level)

    """
    Construct input and output filenames
    """
    if os.path.isdir(args.datadir):
        logging.info(args.datadir)
    else:
        logging.error(args.datadir + ' datadir not found')
        sys.exit(1)
    filename_in = os.path.join(args.datadir, args.input)
    filename_out = os.path.join(args.datadir, args.output)

    """
    Height levels
    """
    levels = np.array([float(lev) for lev in args.levels.split(',')])
    logging.info(('levels', levels))

    """
    Variable list
    """
    var_list = args.vars.split(',')
    logging.info(('vars', var_list))

    """
    Open dataset
    """
    ds = read_tcrmw(filename_in)

    """
    Compute interpolation weights
    """
    compute_interpolation_weights(args, ds, levels)

    """
    Compute tangential and radial wind components
    """
    u_radial, u_tangential = compute_wind_components(args, ds)

    """
    Write dataset
    """
    ds['u_radial'] = xr.DataArray(u_radial, coords=ds[args.T].coords)
    ds['u_tangential'] = xr.DataArray(u_tangential, coords=ds[args.T].coords)
    ds.to_netcdf(filename_out)

    """
    Test plots
    """
    test_plot_pressure_lev(args, ds)



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: eclv_statistics.py
"""
from typing import Union
import warnings
import numpy as np

from metcalcpy.util.ctc_statistics import calculate_economic_value
from metcalcpy.util.utils import sum_column_data_by_name

__author__ = 'Tatiana Burek'


def calculate_eclv(input_data: np.array, columns_names: np.array,
                   thresh: Union[float, None], line_type: str, cl_pts: list, add_base_rate: int = 0) \
        -> Union[dict, None]:
    """Performs calculation of ECLV - The Economic Cost Loss  Value

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            thresh: threshold value for 'pct' line type
            cl_pts: Cost loss ratio. The relative value of being unprepared
                and taking a loss to that of un-necessarily preparing. For example,
                cl = 0.1 indicates it would cost $ 1 to prevent a $10 loss.
                This defaults to the sequence 0.05 to 0.95 by 0.05.
            line_type: line type of the data 'ctc' or 'pct'
            add_base_rate: add Base rate point to cl or not (1 = add, 0 = don't add)

        Returns:
            Returns:
            If assigned to an object, the following values are reported in the dictionary :
                vmax - Maximum value
                V    - Vector of values for each cl value
                F    - Conditional false alarm rate.
                H    - Conditional hit rate
                cl   - Vector of cost loss ratios.
                s    - Base rate
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')

    # some validation
    if line_type != 'ctc' and line_type != 'pct':
        print(f'ERROR: incorrect line type {line_type} for calculating ECLV  ')
        return None
    if line_type == 'pct' and thresh is None:
        print(f'ERROR: provide thresh for calculating ECLV  ')
        return None

    try:
        if line_type == 'pct':
            index_thresh_i = np.where(columns_names == 'thresh_i')[0]
            index_oy_i = np.where(columns_names == 'oy_i')[0]
            index_on_i = np.where(columns_names == 'on_i')[0]
            thresh_i_more = input_data[:, index_thresh_i] > thresh
            thresh_i_less = input_data[:, index_thresh_i] <= thresh

            n11 = np.nansum(input_data[:, index_oy_i][thresh_i_more].astype(np.float))
            n10 = np.nansum(input_data[:, index_on_i][thresh_i_more].astype(np.float))
            n01 = np.nansum(input_data[:, index_oy_i][thresh_i_less].astype(np.float))
            n00 = np.nansum(input_data[:, index_on_i][thresh_i_less].astype(np.float))
        else:
            n11 = sum_column_data_by_name(input_data, columns_names, 'fy_oy')
            n10 = sum_column_data_by_name(input_data, columns_names, 'fy_on')
            n01 = sum_column_data_by_name(input_data, columns_names, 'fn_oy')
            n00 = sum_column_data_by_name(input_data, columns_names, 'fn_on')

        result = calculate_economic_value(np.array([n11, n10, n01, n00]), cl_pts, add_base_rate == 1)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result



import pytest
import numpy as np

from metcalcpy.scorecard import Scorecard, pd


def test_calculate_scorecard_data(settings):
    scorecard = settings['scorecard']
    scorecard.calculate_scorecard_data()
    result_frame = pd.read_csv(
        scorecard.params['sum_stat_output'],
        header=[0],
        sep='\t'
    )
    assert result_frame.size == 72
    assert result_frame.shape == (9, 8)
    assert np.allclose(
    result_frame[(result_frame['model'] == 'DIFF(P200 AFWAOCv3.5.1_d01 120000:240000 HGT BCMSE-P200 NoahMPv3.5.1_d01 120000:240000 HGT BCMSE)')
                 &(result_frame['vx_mask'] == 'EAST')
                 & (result_frame['fcst_lead'] == '120000:240000')]['stat_value'],-6.29828)

    assert np.allclose(
        result_frame[(result_frame[
                          'model'] == 'DIFF_SIG(P200 AFWAOCv3.5.1_d01 360000 HGT BCMSE-P200 NoahMPv3.5.1_d01 360000 HGT BCMSE)')
                     & (result_frame['vx_mask'] == 'NMT')
                     & (result_frame['fcst_lead'] == '120000:240000')]['stat_value'], -0.76703)

    assert np.allclose(
        result_frame[(result_frame[
                          'model'] == 'SINGLE(P200 AFWAOCv3.5.1_d01 480000 HGT BCMSE-P200 NoahMPv3.5.1_d01 480000 HGT BCMSE)')
                     & (result_frame['vx_mask'] == 'EAST')
                     & (result_frame['fcst_lead'] == '480000')]['stat_value'], 439.42705)




@pytest.fixture
def settings():
    """Initialise values for testing.

    Returns:
        dictionary with values of different type
    """
    params = {'append_to_file': False,
              'derived_series_1':
                  [['P200 AFWAOCv3.5.1_d01 120000:240000 HGT BCMSE',
                    'P200 NoahMPv3.5.1_d01 120000:240000 HGT BCMSE',
                    'DIFF'],
                   ['P200 AFWAOCv3.5.1_d01 360000 HGT BCMSE',
                    'P200 NoahMPv3.5.1_d01 360000 HGT BCMSE',
                    'DIFF_SIG'],
                   ['P200 AFWAOCv3.5.1_d01 480000 HGT BCMSE',
                    'P200 NoahMPv3.5.1_d01 480000 HGT BCMSE',
                    'SINGLE']
                   ],
              'equalize_by_indep': True,
              'event_equal': False,
              'fcst_var_val_1': {
                  'HGT':
                      ['BCMSE']},
              'fix_val_list_eq': [],
              'fixed_vars_vals_input': {},
              'indy_plot_val': [],
              'indy_vals': ['EAST',
                            'NMT',
                            'WEST',
                            'G2/TRO'],
              'indy_var': 'vx_mask',
              'line_type': 'sl1l2',
              'list_stat_1':
                  ['BCMSE'],
              'ndays': 10,
              'series_val_1': {
                  'fcst_lead': [
                      '120000:240000',
                      '360000',
                      '480000'],
                  'fcst_lev':
                      ['P200'],
                  'model':
                      ['AFWAOCv3.5.1_d01',
                       'NoahMPv3.5.1_d01']},
              'stat_flag': 'NCAR',
              'sum_stat_input': 'data/scorecard.data',
              'sum_stat_output': 'data/scorecard_output.data'

              }
    scorecard = Scorecard(params)
    settings_dict = dict()
    settings_dict['scorecard'] = scorecard
    return settings_dict



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: compare_images.py
Required packages:
 - pip install scikit-image
 - pip install opencv-python
 - pip install imutils

How to use:
   #initialise the class
   compare = CompareImages('/path/to/image1', '/path/to/image2')
   # get the  mssim value. If vmssim == 1 - images are identical
   mssim = compare.get_ssim()

   compare.save_difference_image(self, '/path/to/image')
   compare.save_difference_image(self, '/path/to/image')
   compare.show_thresh_image(True)
"""
import argparse
import imutils
import cv2
from skimage.metrics import structural_similarity


class CompareImages:
    """A class that performs images comparison using mean structural similarity index over the image (mssim).
        The images are considered equal if mssim = 1
        This class has methods to save or display found differences as an image.
        Currently, the following file formats are supported:
       -   Windows bitmaps - *.bmp, *.dib
       -   JPEG files - *.jpeg, *.jpg, *.jpe
       -   JPEG 2000 files - *.jp2
       -   Portable Network Graphics - *.png
       -   WebP - *.webp
       -   Portable image format - *.pbm, *.pgm, *.ppm *.pxm, *.pnm
       -   Sun rasters - *.sr, *.ras
       -   TIFF files - *.tiff, *.tif
       -   OpenEXR Image files - *.exr
       -   Radiance HDR - *.hdr, *.pic
     """

    def __init__(self, image_path_1, image_path_2):
        """Initialises the class by creating images matrix, mssim
            and the difference image

            Args:
                image_path_1 - path to image 1
                image_path_2 - path to image 2
            Raises:
                an error if on of the images cannot be read
                (because of missing file, improper permissions, unsupported or invalid format)
        """
        self.image_1 = cv2.imread(image_path_1)
        if self.image_1 is None:
            raise KeyError(f'Problems reading the image: {image_path_1}')
        self.image_2 = cv2.imread(image_path_2)
        if self.image_2 is None:
            raise KeyError(f'Problems reading the image: {image_path_2}')

        # convert the images to grayscale
        gray_image_1 = cv2.cvtColor(self.image_1, cv2.COLOR_BGR2GRAY)
        gray_image_2 = cv2.cvtColor(self.image_2, cv2.COLOR_BGR2GRAY)

        # compute the Structural Similarity Index (SSIM) between the two
        # images, ensuring that the difference image is returned
        (self.mssim, self.grad) = structural_similarity(gray_image_1, gray_image_2, full=True)
        self.grad = (self.grad * 255).astype("uint8")
        self.thresh = None
        self.diff_in_image_1 = None
        self.diff_in_image_2 = None

    def get_mssim(self):
        """Returns mean structural similarity index over the image (mssim) as a float
            The images are equal if mssim = 1
        """
        return self.mssim

    def save_difference_image(self, path_to_file):
        """Saves the difference image to the specified file. The image format is chosen based on the
              filename extension
            Args:
                path_to_file - Name of the file
        """
        cv2.imwrite(path_to_file, self.grad)

    def show_difference_image(self, wait_for_key_event=False):
        """ Displays the difference image in the popup window
            Args:
                wait_for_key_event : If True, The function waitKey waits for a key event infinitely
        """
        cv2.imshow("Difference", self.grad)
        if wait_for_key_event:
            cv2.waitKey(0)

    def _init_thresh(self):
        """ The function applies fixed-level thresholding to a multiple-channel array
            to create a threshold image
        """

        # threshold the difference image, followed by finding contours to
        # obtain the regions of the two input images that differ
        self.thresh = cv2.threshold(self.grad, 0, 255,
                                    cv2.THRESH_BINARY_INV | cv2.THRESH_OTSU)[1]

    def save_thresh_image(self, path_to_file):
        """Saves the threshold image to the specified file. The image format is chosen based on the
            filename extension
            Args:
                path_to_file - Name of the file
        """
        if not self.thresh:
            self._init_thresh()
        cv2.imwrite(path_to_file, self.thresh)

    def show_thresh_image(self, wait_for_key_event=False):
        """ Displays the threshold image in the popup window
            Args:
                wait_for_key_event : If True, The function waitKey waits for a key event infinitely
        """
        if not self.thresh:
            self._init_thresh()
        cv2.imshow("Threshold", self.thresh)
        if wait_for_key_event:
            cv2.waitKey(0)

    def _init_diff_in_image_1(self):
        """This function initialises an image with the computed the bounding box
            on the first input images to represent where the two images differ
        """
        if self.thresh is None:
            self._init_thresh()
        contours = cv2.findContours(self.thresh.copy(), cv2.RETR_EXTERNAL,
                                    cv2.CHAIN_APPROX_SIMPLE)
        contours = imutils.grab_contours(contours)
        self.diff_in_image_1 = self.image_1.copy()
        # loop over the contours
        for contour in contours:
            # compute the bounding box of the contour and then draw the
            # bounding box on the first input image to represent where the two
            # images differ
            (x, y, w, h) = cv2.boundingRect(contour)
            cv2.rectangle(self.diff_in_image_1, (x, y), (x + w, y + h), (0, 0, 255), 2)

    def _init_diff_in_image_2(self):
        """This function initialises an image with the computed the bounding box
            on the second input images to represent where the two images differ
        """
        if self.thresh is None:
            self._init_thresh()
        contours = cv2.findContours(self.thresh.copy(), cv2.RETR_EXTERNAL,
                                    cv2.CHAIN_APPROX_SIMPLE)
        contours = imutils.grab_contours(contours)
        self.diff_in_image_2 = self.image_2.copy()
        # loop over the contours
        for contour in contours:
            # compute the bounding box of the contour and then draw the
            # bounding box on the the first input image to represent where the two
            # images differ
            (x, y, w, h) = cv2.boundingRect(contour)
            cv2.rectangle(self.diff_in_image_2, (x, y), (x + w, y + h), (0, 0, 255), 2)

    def save_difference_in_image_1(self, path_to_file):
        """Saves the first image with  the bounding box to the specified file.
            The image format is chosen based on the filename extension
            Args:
                path_to_file - Name of the file
        """
        if self.diff_in_image_1 is None:
            self._init_diff_in_image_1()
        cv2.imwrite(path_to_file, self.diff_in_image_1)

    def show_difference_in_image_1(self, wait_for_key_event=False):
        """ Displays the first image with  the bounding box in the popup window
            Args:
                wait_for_key_event : If True, The function waitKey waits for a key event infinitely
        """
        if self.diff_in_image_1 is None:
            self._init_diff_in_image_1()
        cv2.imshow('Difference in image 1', self.diff_in_image_1)
        if wait_for_key_event:
            cv2.waitKey(0)

    def save_difference_in_image_2(self, path_to_file):
        """Saves the second image with  the bounding box to the specified file.
            The image format is chosen based on the filename extension
            Args:
                path_to_file - Name of the file
        """
        if not self.diff_in_image_2:
            self._init_diff_in_image_2()
        cv2.imwrite(path_to_file, self.diff_in_image_2)

    def show_difference_in_image_2(self, wait_for_key_event=False):
        """ Displays the second image with  the bounding box in the popup window
            Args:
                wait_for_key_event : If True, The function waitKey waits for a key event infinitely
        """
        if not self.diff_in_image_2:
            self._init_diff_in_image_2()
        cv2.imshow('Difference in image 2', self.diff_in_image_2)
        if wait_for_key_event:
            cv2.waitKey(0)


if __name__ == "__main__":
    # construct the argument parse and parse the arguments
    ARG_PARSER = argparse.ArgumentParser()
    ARG_PARSER.add_argument("-f", "--first", required=True,
                            help="first input image")
    ARG_PARSER.add_argument("-s", "--second", required=True,
                            help="second")
    ARGS = vars(ARG_PARSER.parse_args())
    COMPARE = CompareImages(ARGS["first"], ARGS["second"])
    MSSIM = COMPARE.get_mssim()
    if MSSIM == 1.0:
        print("Images are identical")
    else:
        print("Images are different")
        COMPARE.show_difference_image()
        COMPARE.show_thresh_image()
        COMPARE.show_difference_in_image_1()
        COMPARE.show_difference_in_image_2(True)



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Compute OMI index from input OLR data. 
"""

import numpy as np
import xarray as xr
import datetime
import pandas as pd

import compute_mjo_indices as cmi
from plot_mjo_indices import phase_diagram

# set dates to read
datestrt = '1979-01-01'
datelast = '2012-12-31'

spd = 1 # number of obs per day
time = np.arange(datestrt,datelast, dtype='datetime64[D]')
ntim = len(time)

# read OLR from file
ds = xr.open_dataset('/data/mgehne/OLR/olr.1x.7920.nc')
olr = ds['olr'].sel(lat=slice(-20,20),time=slice(datestrt,datelast))
lat = ds['lat'].sel(lat=slice(-20,20))
lon = ds['lon']
print(olr.min(), olr.max())

# project OLR onto EOFs
PC1, PC2 = cmi.omi(olr[0:ntim,:,:], time, spd, './data/')

print(PC1.min(), PC1.max())

# set dates to plot
datestrt = '2012-01-01'
datelast = '2012-03-31'

time = np.arange(datestrt,datelast, dtype='datetime64[D]')
ntim = len(time)
PC1 = PC1.sel(time=slice(datestrt,datelast))
PC2 = PC2.sel(time=slice(datestrt,datelast))
PC1 = PC1[0:ntim]
PC2 = PC2[0:ntim]

months = []
days = []
for idx, val in enumerate(time):
    date = pd.to_datetime(val).timetuple()
    month = date.tm_mon
    day = date.tm_mday
    months.append(month)
    days.append(day)

# plot the PC phase diagram 
phase_diagram('OMI',PC1,PC2,time,months,days,'OMI_comp_phase','png')


# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: read_file.py
Copyright 2021 UCAR/NCAR/RAL
"""

__author__ = 'Hank Fisher'
__version__ = '1.0.0'


import numpy as np
import pandas as pd
import yaml
from read_env_vars_in_config import parse_config
#Two PYTHONPATH directories need to be set for the imports to work
#The parent directory for where METdatadb is installed and
#The METdbload/METdatadb/ush directory for constants in METdbload
from METdatadb import METdbLoad as dbload
from METdatadb.METdbLoad.ush import read_data_files
from METdatadb.METdbLoad.ush.read_load_xml import XmlLoadFile

class ReadMETOutput:

    def __init__(self):
        """ Creates a output reader with a list of files to read
        """
        self.flags = {}
        self.flags['line_type_load'] = False
        self.flags['load_stat'] = True
        self.flags['load_mode'] = True
        self.flags['load_mtd'] = True
        self.flags['load_mpr'] = False
        self.flags['load_orank'] = False
        self.flags['force_dup_file'] = False
        self.flags['verbose'] = False
        self.flags['stat_header_db_check'] = True
        self.flags['tcst_header_db_check'] = True
        self.flags['mode_header_db_check'] = True
        self.flags['mtd_header_db_check'] = True
        self.flags['drop_indexes'] = False
        self.flags['apply_indexes'] = False
        self.flags['load_xml'] = True

    def readYAMLConfig(self,configFile):
        """ Returns a file or list of files

        Args:
            configFile: A YAML formatted config file

        Returns: 
            returns a list containing a single or multiple file names including path
        """
        # Retrieve the contents of a YAML custom config file to over-ride
        # or augment settings defined by the default config file.
        #Use a configure file parser that handles environment variables
        files_dict = parse_config(configFile)

        #parse_config returns a dictionary, read_data_files wants a list
        files = files_dict['files']
        return files

    def readXMLConfig(self,configFile):
        """ Returns a file or list of files
            Args:
                configFile: XML formatted config file
            Returns: 
                returns a list containg a single or multiple file names including path
        """

        # Retrieve the contents of an XML  custom config file to over-ride
        # or augment settings defined by the default config file.
        # Uses XmlLoadFile from METdatadb

        XML_LOADFILE = XmlLoadFile(configFile)
        XML_LOADFILE.read_xml()

        return XML_LOADFILE.load_files

    def readData(self,files_from_config):
        """ 
            Args:
                files_from_config: A list of MET ouptut files grabbed from a config file
            Returns: 
                a pandas DataFrame containing MET output contents
        """

        file_data = read_data_files.ReadDataFiles()

        # read in the data files, with options specified by XML flags
        #set load_flags and line_types empty so that everything is read
        line_types = []
        file_data.read_data(self.flags,
                            files_from_config,
                            line_types)


        df = file_data.stat_data
        return df

def main():
    """
    Reads in a default config file that loads sample MET output data and puts it into a pandas
    DataFrame
    """

    file_reader = ReadMETOutput()

    #Get the files to be loaded either from an XML file
    #Or a YAML file
    #xml_config_file = '../../examples/read_files.xml'
    #load_files = file_reader.readXMLConfig(xml_config_file)

    #The advantage of using YAML is that you can use environment variables to
    #reference a path to the file
    yaml_config_file = "../../examples/read_files.yaml"
    load_files = file_reader.readYAMLConfig(yaml_config_file)

    df = file_reader.readData(load_files)
    print(df)


if __name__ == "__main__":
    main()




# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: sl1l2_statistics.py
"""
import warnings
import numpy as np
from metcalcpy.util.utils import round_half_up, sum_column_data_by_name, PRECISION, get_total_values

__author__ = 'Tatiana Burek'
__version__ = '0.1.0'


def calculate_fbar(input_data, columns_names, aggregation=False):
    """Performs calculation of FBAR - Forecast mean

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated FBAR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        result = sum_column_data_by_name(input_data, columns_names, 'fbar') / total
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_obar(input_data, columns_names, aggregation=False):
    """Performs calculation of OBAR - Observation Mean

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated OBAR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        result = sum_column_data_by_name(input_data, columns_names, 'obar') / total
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_fstdev(input_data, columns_names, aggregation=False):
    """Performs calculation of FSTDEV - Forecast standard deviation

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated FSTDEV as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total1 = 1
        total = sum_column_data_by_name(input_data, columns_names, 'total')
        if aggregation:
            total1 = total
        fbar = sum_column_data_by_name(input_data, columns_names, 'fbar') / total1
        ffbar = sum_column_data_by_name(input_data, columns_names, 'ffbar') / total1
        result = calculate_stddev(fbar * total, ffbar * total, total)
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_ostdev(input_data, columns_names, aggregation=False):
    """Performs calculation of OSTDEV - Observation Standard Deviation

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated OSTDEV as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total1 = 1
        total = sum_column_data_by_name(input_data, columns_names, 'total')
        if aggregation:
            total1 = total
        obar = sum_column_data_by_name(input_data, columns_names, 'obar') / total1
        oobar = sum_column_data_by_name(input_data, columns_names, 'oobar') / total1
        result = calculate_stddev(obar * total, oobar * total, total)
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_fobar(input_data, columns_names, aggregation=False):
    """Performs calculation of FOBAR - Average product of forecast and observation

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated FOBAR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        result = sum_column_data_by_name(input_data, columns_names, 'fobar') / total
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_ffbar(input_data, columns_names, aggregation=False):
    """Performs calculation of FFBAR - Average of forecast squared

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated FFBAR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        result = sum_column_data_by_name(input_data, columns_names, 'ffbar') / total
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_oobar(input_data, columns_names, aggregation=False):
    """Performs calculation of OOBAR - Average of observation squared

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated OOBAR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        result = sum_column_data_by_name(input_data, columns_names, 'oobar') / total
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_mae(input_data, columns_names, aggregation=False):
    """Performs calculation of MAE - Mean absolute error

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated MAE as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        result = sum_column_data_by_name(input_data, columns_names, 'mae') / total
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_mbias(input_data, columns_names, aggregation=False):
    """Performs calculation of MBIAS - Multiplicative Bias

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated MBIAS as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        obar = sum_column_data_by_name(input_data, columns_names, 'obar') / total
        if obar == 0:
            result = None
        else:
            fbar = sum_column_data_by_name(input_data, columns_names, 'fbar') / total
            result = fbar / obar
            result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_pr_corr(input_data, columns_names, aggregation=False):
    """Performs calculation of PR_CORR - Pearson correlation coefficient
        including normal confidence limits

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated PR_CORR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total1 = 1
        total = sum_column_data_by_name(input_data, columns_names, 'total')
        if aggregation:
            total1 = total
        ffbar = sum_column_data_by_name(input_data, columns_names, 'ffbar') / total1
        fbar = sum_column_data_by_name(input_data, columns_names, 'fbar') / total1
        oobar = sum_column_data_by_name(input_data, columns_names, 'oobar') / total1
        obar = sum_column_data_by_name(input_data, columns_names, 'obar') / total1
        fobar = sum_column_data_by_name(input_data, columns_names, 'fobar') / total1
        v = (total ** 2 * ffbar - total ** 2 * fbar ** 2) \
            * (total ** 2 * oobar - total ** 2 * obar ** 2)
        pr_corr = (total ** 2 * fobar - total ** 2 * fbar * obar) / np.sqrt(v)
        if v <= 0 or pr_corr > 1:
            pr_corr = None
        else:
            pr_corr = round_half_up(pr_corr, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        pr_corr = None
    warnings.filterwarnings('ignore')
    return pr_corr


def calculate_fe(input_data, columns_names, aggregation=False):
    """Performs calculation of FE - Fractional error

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated FE as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        fbar = sum_column_data_by_name(input_data, columns_names, 'fbar') / total
        obar = sum_column_data_by_name(input_data, columns_names, 'obar') / total
        result = (fbar - obar) / fbar
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_me(input_data, columns_names, aggregation=False):
    """Performs calculation of ME - Mean error, aka Additive bias

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated ME as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        fbar = sum_column_data_by_name(input_data, columns_names, 'fbar') / total
        obar = sum_column_data_by_name(input_data, columns_names, 'obar') / total
        result = fbar - obar
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_me2(input_data, columns_names, aggregation=False):
    """Performs calculation of ME2 - The square of the mean error (bias)

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated ME2 as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        me = calculate_me(input_data, columns_names, aggregation)
        result = me ** 2
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_mse(input_data, columns_names, aggregation=False):
    """Performs calculation of MSE - Mean squared error

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated MSE as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        ffbar = sum_column_data_by_name(input_data, columns_names, 'ffbar') / total
        oobar = sum_column_data_by_name(input_data, columns_names, 'oobar') / total
        fobar = sum_column_data_by_name(input_data, columns_names, 'fobar') / total
        result = ffbar + oobar - 2 * fobar
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_msess(input_data, columns_names, aggregation=False):
    """Performs calculation of MSESS - The mean squared error skill score

            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
                columns_names: names of the columns for the 2nd dimension as Numpy array
                aggregation: if the aggregation on fields was performed

            Returns:
                calculated MSESS as float
                or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        ostdev = calculate_ostdev(input_data, columns_names, aggregation)
        mse = calculate_mse(input_data, columns_names, aggregation)
        result = 1.0 - mse / ostdev ** 2
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_rmse(input_data, columns_names, aggregation=False):
    """Performs calculation of RMSE - Root-mean squared error

            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
                columns_names: names of the columns for the 2nd dimension as Numpy array
                aggregation: if the aggregation on fields was performed

            Returns:
                calculated RMSE as float
                or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        result = np.sqrt(calculate_mse(input_data, columns_names, aggregation))
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_si(input_data, columns_names, aggregation=False):
    """Performs calculation of SI - Scatter Index

            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
                columns_names: names of the columns for the 2nd dimension as Numpy array
                aggregation: if the aggregation on fields was performed

            Returns:
                calculated SI as float
                or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        rmse = calculate_rmse(input_data, columns_names, aggregation)
        obar = calculate_obar(input_data, columns_names, aggregation)
        result = rmse / obar
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_estdev(input_data, columns_names, aggregation=False):
    """Performs calculation of ESTDEV - Standard deviation of the error

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated ESTDEV as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = sum_column_data_by_name(input_data, columns_names, 'total')
        me = calculate_me(input_data, columns_names)
        mse = calculate_mse(input_data, columns_names)
        result = calculate_stddev(me * total, mse * total, total)
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_bcmse(input_data, columns_names, aggregation=False):
    """Performs calculation of BCMSE - Bias-corrected mean squared error

            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
                columns_names: names of the columns for the 2nd dimension as Numpy array
                aggregation: if the aggregation on fields was performed

            Returns:
                calculated BCMSE as float
                or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        mse = calculate_mse(input_data, columns_names, aggregation)
        me = calculate_me(input_data, columns_names, aggregation)
        result = mse - me ** 2
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_bcrmse(input_data, columns_names, aggregation=False):
    """Performs calculation of BCRMSE - Bias-corrected root mean square error

            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
                columns_names: names of the columns for the 2nd dimension as Numpy array
                aggregation: if the aggregation on fields was performed

            Returns:
                calculated BCRMSE as float
                or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        result = np.sqrt(calculate_bcmse(input_data, columns_names, aggregation))
        result = round_half_up(result, PRECISION)
    except (TypeError, Warning):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calculate_stddev(sum_total, sum_sq, n):
    """Performs calculation of STDDEV - Standard deviation

            Args:
                input_data: 2-dimensional numpy array with data for the calculation
                    1st dimension - the row of data frame
                    2nd dimension - the column of data frame
                columns_names: names of the columns for the 2nd dimension as Numpy array

            Returns:
                calculated STDDEV as float
                or None if some of the data values are missing or invalid
    """
    if n < 1:
        return None
    v = (sum_sq - sum_total * sum_total / n) / (n - 1)
    if v < 0:
        return None

    return np.sqrt(v)


def calculate_sl1l2_total(input_data, columns_names):
    """Performs calculation of Total number of matched pairs for
        Scalar Partial Sums
        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated Total number of matched pairs as float
            or None if some of the data values are missing or invalid
    """
    total = sum_column_data_by_name(input_data, columns_names, 'total')
    return round_half_up(total, PRECISION)


def calculate_sal1l2_total(input_data, columns_names):
    """Performs calculation of Total number of matched pairs for
        Scalar Anomaly Partial Sums
        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated Total number of matched pairs as float
            or None if some of the data values are missing or invalid
    """
    total = sum_column_data_by_name(input_data, columns_names, 'total')
    return round_half_up(total, PRECISION)



# ============================*
 # ** Copyright UCAR (c) 2020
 # ** University Corporation for Atmospheric Research (UCAR)
 # ** National Center for Atmospheric Research (NCAR)
 # ** Research Applications Lab (RAL)
 # ** P.O.Box 3000, Boulder, Colorado, 80307-3000, USA
 # ============================*
 
 
 
"""
Program Name: val1l2_statistics.py
"""
import warnings
import numpy as np

from metcalcpy.util.utils import round_half_up, sum_column_data_by_name, PRECISION, get_total_values

__author__ = 'Tatiana Burek'
__version__ = '0.1.0'


def calculate_val1l2_anom_corr(input_data, columns_names, aggregation=False):
    """Performs calculation of VAL1L2_ANOM_CORR -

        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array
            aggregation: if the aggregation on fields was performed

        Returns:
            calculated VAL1L2_ANOM_CORR as float
            or None if some of the data values are missing or invalid
    """
    warnings.filterwarnings('error')
    try:
        total = get_total_values(input_data, columns_names, aggregation)
        ufabar = sum_column_data_by_name(input_data, columns_names, 'ufabar') / total
        vfabar = sum_column_data_by_name(input_data, columns_names, 'vfabar') / total
        uoabar = sum_column_data_by_name(input_data, columns_names, 'uoabar') / total
        voabar = sum_column_data_by_name(input_data, columns_names, 'voabar') / total
        uvfoabar = sum_column_data_by_name(input_data, columns_names, 'uvfoabar') / total
        uvffabar = sum_column_data_by_name(input_data, columns_names, 'uvffabar') / total
        uvooabar = sum_column_data_by_name(input_data, columns_names, 'uvooabar') / total
        result = calc_wind_corr(ufabar, vfabar, uoabar, voabar, uvfoabar, uvffabar, uvooabar)
        result = round_half_up(result, PRECISION)
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        result = None
    warnings.filterwarnings('ignore')
    return result


def calc_wind_corr(uf, vf, uo, vo, uvfo, uvff, uvoo):
    """Calculates  wind correlation
        Args:
            uf - Mean(uf-uc)
            vf - Mean(vf-vc)
            uo - Mean(uo-uc)
            vo - Mean(vo-vc)
            uvfo - Mean((uf-uc)*(uo-uc)+(vf-vc)*(vo-vc))
            uvff - Mean((uf-uc)^2+(vf-vc)^2)
            uvoo - Mean((uo-uc)^2+(vo-vc)^2)

        Returns:
                calculated wind correlation as float
                or None if some of the data values are None
        """
    try:
        corr = (uvfo - uf * uo - vf * vo) / (np.sqrt(uvff - uf * uf - vf * vf)
                                             * np.sqrt(uvoo - uo * uo - vo * vo))
    except (TypeError, ZeroDivisionError, Warning, ValueError):
        corr = None
    return corr


def calculate_val1l2_total(input_data, columns_names):
    """Performs calculation of Total number of matched pairs for
        Vector Anomaly Partial Sums
        Args:
            input_data: 2-dimensional numpy array with data for the calculation
                1st dimension - the row of data frame
                2nd dimension - the column of data frame
            columns_names: names of the columns for the 2nd dimension as Numpy array

        Returns:
            calculated Total number of matched pairs as float
            or None if some of the data values are missing or invalid
    """
    total = sum_column_data_by_name(input_data, columns_names, 'total')
    return round_half_up(total, PRECISION)
